'#################################################################################################
'#
'#   Yahoo Socket Class
'#
'#   Purpose: To communicate over Yahoo Chat service.
'#
'#   Written by Xeon Productions
'#   Copyright 2004 Xeon Productions
'#   Contact: Brandon Scott [xeons@xeons.net]
'#   Website: http://www.xeons.net/
'# 
'#   If you use this code please ask permission first, or give
'#   credit where credit is due.
'#
'#################################################################################################

Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Text.ASCIIEncoding
Imports System.Text.RegularExpressions

#Region "Sockets Client"
Public Class SocketClient
    Public Event OnConnect()
    Public Event OnError(ByVal Description As String)
    Public Event OnDataArrival(ByVal Data As String, ByVal TotalBytes As Integer)
    Public Event OnDisconnect()
    Public Event OnSendComplete(ByVal DataSize As Integer)

    'Private response As [String] = [String].Empty
    Private port As Integer = 44
    Private ipHostInfo As IPHostEntry = Dns.Resolve("localhost")
    Private ipAddress As ipAddress = ipHostInfo.AddressList(0)
    Private client As New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)

    Public BufferSize As Integer = 32767
    Public Buffer(32767) As Byte

    Public Sub Connect(Optional ByVal RemoteHostName As String = "scs.msg.yahoo.com", _
    Optional ByVal RemotePort As Integer = 5050)
        Try
            client = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
            port = RemotePort
            ipHostInfo = Dns.Resolve(RemoteHostName)
            ipAddress = ipHostInfo.AddressList(0)
            Dim remoteEP As New IPEndPoint(ipAddress, port)
            client.BeginConnect(remoteEP, AddressOf sockConnected, Nothing)
        Catch
            RaiseEvent OnError("CONNECT " & Err.Description)
            Exit Sub
        End Try
    End Sub

    Public Sub SendData(ByVal Data() As Byte)
        Try
            Dim byteData As Byte() = Data
            client.BeginSend(byteData, 0, byteData.Length, 0, AddressOf sockSendEnd, Nothing)
        Catch
            RaiseEvent OnError("SENDDATA1 " & Err.Description)
            Exit Sub
        End Try
    End Sub

    Public Sub SendData(ByVal Data As String)
        Try
            Dim byteData As Byte() = Encoding.Default.GetBytes(Data)
            client.BeginSend(byteData, 0, byteData.Length, 0, AddressOf sockSendEnd, Nothing)
        Catch
            RaiseEvent OnError("SENDDATA2 " & Err.Description)
            Exit Sub
        End Try
    End Sub

    Public Sub Disconnect()
        Try
            client.Shutdown(SocketShutdown.Both)
        Catch
        End Try
        client.Close()
    End Sub

    Private Sub sockConnected(ByVal ar As IAsyncResult)
        Try
            If client.Connected = False Then RaiseEvent OnError("Connection refused.") : Exit Sub
            client.BeginReceive(Buffer, 0, BufferSize, 0, AddressOf sockDataArrival, Nothing)
            RaiseEvent OnConnect()
        Catch
            RaiseEvent OnError("CONNECTED " & Err.Description)
            Exit Sub
        End Try
    End Sub

    Private Sub sockDataArrival(ByVal ar As IAsyncResult)
        Dim locBuffer() As Byte
        Dim bytesRead As Integer = 0
        Try
            bytesRead = client.EndReceive(ar)
            If bytesRead = 0 Then
                client.Shutdown(SocketShutdown.Both)
                client.Close()
                RaiseEvent OnDisconnect()
                Exit Sub
            Else
                locBuffer = Buffer
                ReDim Buffer(BufferSize)
                client.BeginReceive(Buffer, 0, BufferSize, SocketFlags.None, New AsyncCallback(AddressOf sockDataArrival), Nothing)
            End If
        Catch ex As Exception
            RaiseEvent OnError("DATA-ARRIVAL " & ex.Message)
            Exit Sub
        End Try
        RaiseEvent OnDataArrival(Encoding.Default.GetString(locBuffer).Substring(0, bytesRead), bytesRead)
    End Sub

    Private Sub sockSendEnd(ByVal ar As IAsyncResult)
        Try
            'Dim client As Socket = CType(ar.AsyncState, Socket)
            Dim bytesSent As Integer = client.EndSend(ar)
            RaiseEvent OnSendComplete(bytesSent)
        Catch
            RaiseEvent OnError("SENDEND " & Err.Description)
            Exit Sub
        End Try
    End Sub

    Public Function Connected() As Boolean
        Try
            Return client.Connected
        Catch
            RaiseEvent OnError("CONNECTED " & Err.Description)
            Exit Function
        End Try
    End Function
End Class
#End Region
#Region "Yahoo Socket"
Public Class YahooSocket
    Private WithEvents YSocket As New SocketClient

    Private m_YahooID As String
    Private m_Password As String
    Private m_Challenge As String
    Private m_ChallengeResults(1) As String
    Private m_SessionID As Long
    Private m_Status As Long
    Private m_Cookie As String
    Private m_ErrorCode As Integer
    Private m_ErrorURL As String
    Private m_BuddyListData As String
    Private m_From As String
    Private m_UTF8 As Boolean
    Private m_BuddyIcon As Boolean
    Private m_Message As String

    Private IncomingBuffer As New StringBuilder
    Private PacketLength As Integer
    Private DataPending As Boolean

    Public Event OnReceivedVerify()
    Public Event OnReceivedChallangeRequest(ByVal Challange As String)
    Public Event OnLoginFailure(ByVal ErrorCode As Integer, ByVal ErrorMessage As String)
    Public Event OnReceivedBuddylist(ByVal Groupname As String, ByVal Buddies As ArrayList)
    Public Event OnReceivedInstantMessage(ByVal YahooID As String, ByVal Message As String, ByVal HasBuddyicon As Boolean, ByVal IsUTF8Encoded As Boolean)
    Public Event OnReceivedError(ByVal Message As String)
    Public Event OnBuddyListPacket(ByVal Group As String, ByVal Buddies() As String)

    Private Sub YSocket_OnDataArrival(ByVal Data As String, ByVal TotalBytes As Integer) Handles YSocket.OnDataArrival
        IncomingBuffer.Append(Data)
NextPacket:
        If DataPending = False Then
            PacketLength = GetWord(Mid(IncomingBuffer.ToString, 9, 2)) + 20
        Else
            PacketLength = GetWord(Mid(IncomingBuffer.ToString, 9, 2)) + 20
            If PacketLength > IncomingBuffer.Length Then
                DataPending = True
                Exit Sub
            End If
        End If
        If PacketLength > IncomingBuffer.Length Then
            DataPending = True
            Exit Sub
        End If
        Call ProcessData(Mid(IncomingBuffer.ToString, 1, PacketLength))
        IncomingBuffer.Remove(0, PacketLength)
        PacketLength = 0
        DataPending = False
        If IncomingBuffer.Length >= 20 Then GoTo NextPacket
    End Sub

    Private Sub YSocket_OnConnect() Handles YSocket.OnConnect
        Dim Packet As New YahooPacket(YMSG_CMD_VERIFY, 0, 0)
        YSocket.SendData(Packet.ToString)
    End Sub

    Private Sub ProcessData(ByVal Data As String)
        Dim YData As New YahooPacket(Data)
        Dim en As IDictionaryEnumerator = YData.Pairs.GetEnumerator
        Dim ErrorCode As Integer
        Dim i As Integer, j As Integer

        Try
            Select Case YData.Command

                Case YMSG_CMD_VERIFY

                    RaiseEvent OnReceivedVerify()
                    Call SendChallengeRequest()

                Case YMSG_CMD_AUTH

                    While en.MoveNext
                        If en.Key = 1 Then
                            m_YahooID = en.Value
                        ElseIf en.Key = 94 Then
                            m_Challenge = en.Value
                        End If
                    End While
                    m_SessionID = YData.SessionID
                    RaiseEvent OnReceivedChallangeRequest(m_Challenge)
                    Call SendLoginRequest()

                Case YMSG_CMD_AUTHRESP

                    While en.MoveNext
                        If en.Key = 66 Then
                            m_ErrorCode = en.Value
                        ElseIf en.Key = 20 Then
                            m_ErrorURL = en.Value
                        End If
                    End While
                    Select Case m_ErrorCode
                        Case 3
                            RaiseEvent OnLoginFailure(m_ErrorCode, "Invalid username.")
                        Case 13
                            RaiseEvent OnLoginFailure(m_ErrorCode, "Incorrect password, or Yahoo's authentication method has changed once again!")
                        Case 14
                            RaiseEvent OnLoginFailure(m_ErrorCode, "Your account is locked, please log in to the Yahoo! website.")
                        Case Else
                            RaiseEvent OnLoginFailure(m_ErrorCode, "Unknown error number " & ErrorCode & ".")
                    End Select

                Case YMSG_CMD_MESSAGE, YMSG_CMD_GAMEMSG, YMSG_CMD_CHATMSG

                    If YData.Status <= 1 Or YData.Status = 5 Then
                        While en.MoveNext
                            If en.Key = 4 Then
                                m_From = en.Value
                            ElseIf en.Key = 97 Then
                                m_UTF8 = en.Value
                            ElseIf en.Key = 206 Then
                                m_BuddyIcon = en.Value
                            ElseIf en.Key = 14 Then
                                m_Message = en.Value
                            End If
                        End While
                    ElseIf YData.Status = 2 Then
                        RaiseEvent OnReceivedError("Your Yahoo! message did not get sent.")
                        Exit Sub
                    End If
                    m_Message = DecodeYahooString(m_Message, m_UTF8)
                    Debug.WriteLine(m_Message.IndexOf("<ding>"))
                    If m_Message.IndexOf("<ding>") = -1 Then
                        RaiseEvent OnReceivedInstantMessage(m_From, m_Message, m_BuddyIcon, m_UTF8)
                    Else
                        RaiseEvent OnReceivedInstantMessage(m_From, "Buzz!!", m_BuddyIcon, m_UTF8)
                    End If

                Case YMSG_CMD_LIST


                    Dim strTmpYBList() As String
                    Dim strTmpYBListBuddies() As String
                    While en.MoveNext
                        If en.Key = 87 Then
                            m_BuddyListData = en.Value
                            strTmpYBList = m_BuddyListData.Split(":")
                            For i = 0 To strTmpYBList.Length - 1 Step 2
                                strTmpYBListBuddies = strTmpYBList(i + 1).Split(",")
                                RaiseEvent OnReceivedBuddylist(strTmpYBList(i), New ArrayList(strTmpYBListBuddies))
                            Next i
                        End If
                    End While

                    Debug.WriteLine(YData.ToString.Replace(Chr(0), "\0"))

            End Select
        Catch ex As Exception
            Debug.WriteLine("ProcessData was gay!: " & ex.ToString)
        End Try
    End Sub

    Public Sub Login(ByVal YahooID As String, ByVal Password As String, Optional ByVal HostName As String = "scs.msg.yahoo.com", Optional ByVal Port As Integer = 5050)
        m_YahooID = YahooID.Replace(" ", "").ToLower
        m_Password = Password
        YSocket.Disconnect()
        YSocket.Connect(HostName, Port)
    End Sub

    Public Sub Logout()
        m_YahooID = ""
        m_Password = ""
        YSocket.Disconnect()
    End Sub

    Public Sub SendInstantMessage(ByVal YahooID As String, ByVal Message As String)
        Dim Packet As New YahooPacket(YMSG_CMD_MESSAGE, YMSG_STATUS_OFFLINE, m_SessionID)
        Message = EncodeYahooString(Message, True)
        Packet.AddPair(1, m_YahooID)
        Packet.AddPair(5, YahooID)
        Packet.AddPair(14, Message)
        Packet.AddPair(97, "1")
        Packet.AddPair(63, ";0")
        Packet.AddPair(64, "0")
        Packet.AddPair(1002, "1")
        Packet.AddPair(206, "0")
        YSocket.SendData(Packet.ToString)
    End Sub

    Public Sub SendBuzz(ByVal YahooID As String)
        Call SendInstantMessage(YahooID, "<ding>")
    End Sub

    Private Sub SendChallengeRequest()
        Dim Packet As New YahooPacket(YMSG_CMD_AUTH, 0, 0)
        Packet.AddPair(1, m_YahooID)
        YSocket.SendData(Packet.ToString)
    End Sub

    Private Sub SendLoginRequest()
        Dim Packet As New YahooPacket(YMSG_CMD_AUTHRESP, YMSG_STATUS_AVAILABLE, m_SessionID)
        Dim YChallenge As New YahooChallengeResponse
        YChallenge.Username = m_YahooID
        YChallenge.Password = m_Password
        YChallenge.Challenge = m_Challenge
        m_ChallengeResults = YChallenge.GetLoginStrings()
        Packet.AddPair(1, m_YahooID)
        Packet.AddPair(6, m_ChallengeResults(0))
        Packet.AddPair(96, m_ChallengeResults(1))
        Packet.AddPair(0, m_YahooID)
        YSocket.SendData(Packet.ToString)
    End Sub

End Class
#End Region